@@ -4,9 +4,6 @@
 
 #pragma once
 
-#include <OpenImageIO/filesystem.h>
-#include <OpenImageIO/fmath.h>
-
 
 OIIO_PLUGIN_NAMESPACE_BEGIN
 
