@@ -767,7 +767,11 @@ struct GifWriter
 bool GifBegin( GifWriter* writer, const char* filename, uint32_t width, uint32_t height, uint32_t delay, int32_t bitDepth = 8, bool dither = false )
 {
     (void)bitDepth; (void)dither; // Mute "Unused argument" warnings
-#if defined(_MSC_VER) && (_MSC_VER >= 1400)
+
+#ifdef OIIO_FILESYSTEM_H
+    // If available, use OIIO's UTF8-safe fopen
+    writer->f = OIIO::Filesystem::fopen(filename, "wb");
+#elif defined(_MSC_VER) && (_MSC_VER >= 1400)
 	writer->f = 0;
     fopen_s(&writer->f, filename, "wb");
 #else
