@@ -52,16 +52,20 @@ static bool compute_stats = false;
 static void
 print_sha1(ImageInput* input)
 {
+    using Strutil::printf;
     SHA1 sha;
     const ImageSpec& spec(input->spec());
     if (spec.deep) {
         // Special handling of deep data
         DeepData dd;
         if (!input->read_native_deep_image(dd)) {
-            printf("    SHA-1: unable to compute, could not read image\n");
+            std::string err = input->geterror();
+            if (err.empty())
+                err = "could not read image";
+            printf("    SHA-1: %s\n", err);
             return;
         }
-        // Hash both the sample counds and the data block
+        // Hash both the sample counts and the data block
         sha.append(dd.all_samples());
         sha.append(dd.all_data());
     } else {
@@ -72,7 +76,10 @@ print_sha1(ImageInput* input)
         }
         std::unique_ptr<char[]> buf(new char[size]);
         if (!input->read_image(TypeDesc::UNKNOWN /*native*/, &buf[0])) {
-            printf("    SHA-1: unable to compute, could not read image\n");
+            std::string err = input->geterror();
+            if (err.empty())
+                err = "could not read image";
+            printf("    SHA-1: %s\n", err);
             return;
         }
         sha.append(&buf[0], size);
@@ -614,16 +621,6 @@ print_info(const std::string& filename, size_t namefieldlength,
 
 
 
-static int
-parse_files(int argc, const char* argv[])
-{
-    for (int i = 0; i < argc; i++)
-        filenames.emplace_back(argv[i]);
-    return 0;
-}
-
-
-
 int
 main(int argc, const char* argv[])
 {
@@ -634,23 +631,32 @@ main(int argc, const char* argv[])
     Filesystem::convert_native_arguments(argc, (const char**)argv);
     ArgParse ap;
     // clang-format off
-    ap.options ("iinfo -- print information about images\n"
-                OIIO_INTRO_STRING "\n"
-                "Usage:  iinfo [options] filename...",
-                "%*", parse_files, "",
-                "--help", &help, "Print help message",
-                "-v", &verbose, "Verbose output",
-                "-m %s:NAMES", &metamatch, "Metadata names to print (default: all)",
-                "-f", &filenameprefix, "Prefix each line with the filename",
-                "-s", &sum, "Sum the image sizes",
-                "-a", &subimages, "Print info about all subimages",
-                "--hash", &compute_sha1, "Print SHA-1 hash of pixel values",
-                "--stats", &compute_stats, "Print image pixel statistics (data window)",
-                NULL);
+    ap.intro("iinfo -- print information about images\n"
+             OIIO_INTRO_STRING);
+    ap.usage("iinfo [options] filename...");
+    ap.arg("filename")
+      .hidden()
+      .action([&](cspan<const char*> argv){ filenames.emplace_back(argv[0]); });
+    ap.arg("-v", &verbose)
+      .help("Verbose output");
+    ap.arg("-m %s:NAMES", &metamatch)
+      .help("Metadata names to print (default: all)");
+    ap.arg("-f", &filenameprefix)
+      .help("Prefix each line with the filename");
+    ap.arg("-s", &sum)
+      .help("Sum the image sizes");
+    ap.arg("-a", &subimages)
+      .help("Print info about all subimages")
+      .action(ArgParse::store_true());
+    ap.arg("--hash", &compute_sha1)
+      .help("Print SHA-1 hash of pixel values")
+      .action(ArgParse::store_true());
+    ap.arg("--stats", &compute_stats)
+      .help("Print image pixel statistics (data window)");
     // clang-format on
     if (ap.parse(argc, argv) < 0 || filenames.empty()) {
         std::cerr << ap.geterror() << std::endl;
-        ap.usage();
+        ap.print_help();
         return help ? EXIT_SUCCESS : EXIT_FAILURE;
     }
 
