@@ -26,6 +26,12 @@
 #include <OpenImageIO/export.h>
 #include <OpenImageIO/oiioversion.h>
 #include <OpenImageIO/string_view.h>
+#include <OpenImageIO/strutil.h>
+
+// Define symbols that let client applications determine if newly added
+// features are supported.
+#define OIIO_TYPEDESC_VECTOR2 1
+
 
 
 OIIO_NAMESPACE_BEGIN
@@ -111,7 +117,8 @@ struct OIIO_API TypeDesc {
                           ///<   4-byte encoding of an SMPTE timecode.
         KEYCODE,          ///< indicates an `int[7]` representing the standard
                           ///<   28-byte encoding of an SMPTE keycode.
-        RATIONAL          ///< A VEC2 representing a rational number `val[0] / val[1]`
+        RATIONAL,         ///< A VEC2 representing a rational number `val[0] / val[1]`
+        BOX,              ///< A VEC2[2] or VEC3[2] that represents a 2D or 3D bounds (min/max)
     };
 
     unsigned char basetype;      ///< C data type at the heart of our type
@@ -166,7 +173,7 @@ struct OIIO_API TypeDesc {
     /// "float", "int[5]", "normal"
     const char *c_str() const;
 
-    friend std::ostream& operator<< (std::ostream& o, TypeDesc t) {
+    friend std::ostream& operator<< (std::ostream& o, const TypeDesc& t) {
         o << t.c_str();  return o;
     }
 
@@ -222,9 +229,9 @@ struct OIIO_API TypeDesc {
     /// ignoring whether it's an array).
     size_t elementsize () const noexcept { return aggregate * basesize(); }
 
-    // /// Return just the underlying C scalar type, i.e., strip out the
-    // /// array-ness and the aggregateness.
-//    BASETYPE basetype () const { return TypeDesc(base); }
+    /// Return just the underlying C scalar type, i.e., strip out the
+    /// array-ness and the aggregateness.
+    constexpr TypeDesc scalartype() const { return TypeDesc(BASETYPE(basetype)); }
 
     /// Return the base type size, i.e., stripped of both array-ness
     /// and aggregateness.
@@ -240,7 +247,7 @@ struct OIIO_API TypeDesc {
     /// Shortcut: is it UNKNOWN?
     constexpr bool is_unknown () const noexcept { return (basetype == UNKNOWN); }
 
-    /// if (typespec) is the same as asking whether it's not UNKNOWN.
+    /// if (typedesc) is the same as asking whether it's not UNKNOWN.
     constexpr operator bool () const noexcept { return (basetype != UNKNOWN); }
 
     /// Set *this to the type described in the string.  Return the
@@ -292,6 +299,11 @@ struct OIIO_API TypeDesc {
                                              || (this->is_sized_array()   && b.is_unsized_array()));
     }
 
+    /// Is this a 2-vector aggregate (of the given type, float by default)?
+    constexpr bool is_vec2 (BASETYPE b=FLOAT) const noexcept {
+        return this->aggregate == VEC2 && this->basetype == b && !is_array();
+    }
+
     /// Is this a 3-vector aggregate (of the given type, float by default)?
     constexpr bool is_vec3 (BASETYPE b=FLOAT) const noexcept {
         return this->aggregate == VEC3 && this->basetype == b && !is_array();
@@ -302,6 +314,18 @@ struct OIIO_API TypeDesc {
         return this->aggregate == VEC4 && this->basetype == b && !is_array();
     }
 
+    /// Is this an array of aggregates that represents a 2D bounding box?
+    constexpr bool is_box2 (BASETYPE b=FLOAT) const noexcept {
+        return this->aggregate == VEC2 && this->basetype == b && arraylen == 2
+                && this->vecsemantics == BOX;
+    }
+
+    /// Is this an array of aggregates that represents a 3D bounding box?
+    constexpr bool is_box3 (BASETYPE b=FLOAT) const noexcept {
+        return this->aggregate == VEC3 && this->basetype == b && arraylen == 2
+                && this->vecsemantics == BOX;
+    }
+
     /// Demote the type to a non-array
     ///
     void unarray (void) noexcept { arraylen = 0; }
@@ -310,6 +334,11 @@ struct OIIO_API TypeDesc {
     /// containers and algorithms.
     bool operator< (const TypeDesc &x) const noexcept;
 
+    /// Given base data types of a and b, return a basetype that is a best
+    /// guess for one that can handle both without any loss of range or
+    /// precision.
+    static BASETYPE basetype_merge(TypeDesc a, TypeDesc b);
+
     // DEPRECATED(1.8): These static const member functions were mildly
     // problematic because they required external linkage (and possibly
     // even static initialization order fiasco) and were a memory reference
@@ -348,6 +377,10 @@ static constexpr TypeDesc TypeNormal (TypeDesc::FLOAT, TypeDesc::VEC3, TypeDesc:
 static constexpr TypeDesc TypeMatrix33 (TypeDesc::FLOAT, TypeDesc::MATRIX33);
 static constexpr TypeDesc TypeMatrix44 (TypeDesc::FLOAT, TypeDesc::MATRIX44);
 static constexpr TypeDesc TypeMatrix = TypeMatrix44;
+static constexpr TypeDesc TypeFloat2 (TypeDesc::FLOAT, TypeDesc::VEC2);
+static constexpr TypeDesc TypeVector2 (TypeDesc::FLOAT, TypeDesc::VEC2, TypeDesc::VECTOR);
+static constexpr TypeDesc TypeFloat4 (TypeDesc::FLOAT, TypeDesc::VEC4);
+static constexpr TypeDesc TypeVector4 = TypeFloat4;
 static constexpr TypeDesc TypeString (TypeDesc::STRING);
 static constexpr TypeDesc TypeInt (TypeDesc::INT);
 static constexpr TypeDesc TypeUInt (TypeDesc::UINT);
@@ -357,15 +390,22 @@ static constexpr TypeDesc TypeInt16 (TypeDesc::INT16);
 static constexpr TypeDesc TypeUInt16 (TypeDesc::UINT16);
 static constexpr TypeDesc TypeInt8 (TypeDesc::INT8);
 static constexpr TypeDesc TypeUInt8 (TypeDesc::UINT8);
+static constexpr TypeDesc TypeInt64 (TypeDesc::INT64);
+static constexpr TypeDesc TypeUInt64 (TypeDesc::UINT64);
+static constexpr TypeDesc TypeVector2i(TypeDesc::INT, TypeDesc::VEC2);
+static constexpr TypeDesc TypeBox2(TypeDesc::FLOAT, TypeDesc::VEC2, TypeDesc::BOX, 2);
+static constexpr TypeDesc TypeBox3(TypeDesc::FLOAT, TypeDesc::VEC3, TypeDesc::BOX, 2);
+static constexpr TypeDesc TypeBox2i(TypeDesc::INT, TypeDesc::VEC2, TypeDesc::BOX, 2);
+static constexpr TypeDesc TypeBox3i(TypeDesc::INT, TypeDesc::VEC3, TypeDesc::BOX, 2);
 static constexpr TypeDesc TypeHalf (TypeDesc::HALF);
 static constexpr TypeDesc TypeTimeCode (TypeDesc::UINT, TypeDesc::SCALAR, TypeDesc::TIMECODE, 2);
 static constexpr TypeDesc TypeKeyCode (TypeDesc::INT, TypeDesc::SCALAR, TypeDesc::KEYCODE, 7);
-static constexpr TypeDesc TypeFloat4 (TypeDesc::FLOAT, TypeDesc::VEC4);
 static constexpr TypeDesc TypeRational(TypeDesc::INT, TypeDesc::VEC2, TypeDesc::RATIONAL);
+static constexpr TypeDesc TypePointer(TypeDesc::PTR);
 
 
 
-// DEPRECATED
+// DEPRECATED(2.1)
 OIIO_API
 std::string tostring (TypeDesc type, const void *data,
                       const char *float_fmt,                // E.g. "%g"
@@ -388,7 +428,7 @@ template<> struct BaseTypeFromC<unsigned int> { static const TypeDesc::BASETYPE
 template<> struct BaseTypeFromC<int> { static const TypeDesc::BASETYPE value = TypeDesc::INT; };
 template<> struct BaseTypeFromC<unsigned long long> { static const TypeDesc::BASETYPE value = TypeDesc::UINT64; };
 template<> struct BaseTypeFromC<long long> { static const TypeDesc::BASETYPE value = TypeDesc::INT64; };
-#ifdef _HALF_H_
+#if defined(_HALF_H_) || defined(IMATH_HALF_H_)
 template<> struct BaseTypeFromC<half> { static const TypeDesc::BASETYPE value = TypeDesc::HALF; };
 #endif
 template<> struct BaseTypeFromC<float> { static const TypeDesc::BASETYPE value = TypeDesc::FLOAT; };
@@ -413,6 +453,9 @@ template<size_t S> struct TypeDescFromC<char[S]> { static const constexpr TypeDe
 template<size_t S> struct TypeDescFromC<const char[S]> { static const constexpr TypeDesc value() { return TypeDesc::STRING; } };
 #ifdef INCLUDED_IMATHVEC_H
 template<> struct TypeDescFromC<Imath::V3f> { static const constexpr TypeDesc value() { return TypeVector; } };
+template<> struct TypeDescFromC<Imath::V2f> { static const constexpr TypeDesc value() { return TypeVector2; } };
+template<> struct TypeDescFromC<Imath::V4f> { static const constexpr TypeDesc value() { return TypeVector4; } };
+template<> struct TypeDescFromC<Imath::V2i> { static const constexpr TypeDesc value() { return TypeVector2i; } };
 #endif
 #ifdef INCLUDED_IMATHCOLOR_H
 template<> struct TypeDescFromC<Imath::Color3f> { static const constexpr TypeDesc value() { return TypeColor; } };
@@ -420,6 +463,14 @@ template<> struct TypeDescFromC<Imath::Color3f> { static const constexpr TypeDes
 #ifdef INCLUDED_IMATHMATRIX_H
 template<> struct TypeDescFromC<Imath::M33f> { static const constexpr TypeDesc value() { return TypeMatrix33; } };
 template<> struct TypeDescFromC<Imath::M44f> { static const constexpr TypeDesc value() { return TypeMatrix44; } };
+template<> struct TypeDescFromC<Imath::M33d> { static const constexpr TypeDesc value() { return TypeDesc(TypeDesc::DOUBLE, TypeDesc::MATRIX33); } };
+template<> struct TypeDescFromC<Imath::M44d> { static const constexpr TypeDesc value() { return TypeDesc(TypeDesc::DOUBLE, TypeDesc::MATRIX44); } };
+#endif
+#ifdef INCLUDED_IMATHBOX_H
+template<> struct TypeDescFromC<Imath::Box2f> { static const constexpr TypeDesc value() { return TypeBox2; } };
+template<> struct TypeDescFromC<Imath::Box2i> { static const constexpr TypeDesc value() { return TypeBox2i; } };
+template<> struct TypeDescFromC<Imath::Box3f> { static const constexpr TypeDesc value() { return TypeBox3; } };
+template<> struct TypeDescFromC<Imath::Box3i> { static const constexpr TypeDesc value() { return TypeBox3i; } };
 #endif
 
 
@@ -434,7 +485,7 @@ template<> struct CType<(int)TypeDesc::UINT> { typedef unsigned int type; };
 template<> struct CType<(int)TypeDesc::INT> { typedef int type; };
 template<> struct CType<(int)TypeDesc::UINT64> { typedef unsigned long long type; };
 template<> struct CType<(int)TypeDesc::INT64> { typedef long long type; };
-#ifdef _HALF_H_
+#if defined(_HALF_H_) || defined(IMATH_HALF_H_)
 template<> struct CType<(int)TypeDesc::HALF> { typedef half type; };
 #endif
 template<> struct CType<(int)TypeDesc::FLOAT> { typedef float type; };
@@ -444,7 +495,8 @@ template<> struct CType<(int)TypeDesc::DOUBLE> { typedef double type; };
 
 /// Helper class for tostring() that contains a whole bunch of parameters
 /// that control exactly how all the data types that can be described as
-/// TypeDesc ought to be formatted as a string.
+/// TypeDesc ought to be formatted as a string. Uses printf-like
+/// conventions. This will someday be deprecated.
 struct tostring_formatting {
     // Printf-like formatting specs for int, float, string, pointer data.
     const char *int_fmt = "%d";
@@ -465,9 +517,12 @@ struct tostring_formatting {
     enum Flags { None=0, escape_strings=1, quote_single_string=2 };
     int flags = escape_strings;
     // Reserved space for future expansion without breaking the ABI.
-    const char *reserved1 = "";
+    const char *uint_fmt = "%u";
     const char *reserved2 = "";
     const char *reserved3 = "";
+    bool use_sprintf = true;
+
+    enum Notation { STDFORMAT };
 
     tostring_formatting() = default;
     tostring_formatting(const char *int_fmt, const char *float_fmt = "%g",
@@ -475,13 +530,26 @@ struct tostring_formatting {
         const char *aggregate_begin = "(", const char *aggregate_end = ")",
         const char *aggregate_sep = ",", const char *array_begin = "{",
         const char *array_end = "}", const char *array_sep = ",",
+        int flags = escape_strings,
+        const char *uint_fmt = "%u");
+
+    // Alternative ctr for std::format notation. You must pass STDFORMAT
+    // as the first argument.
+    tostring_formatting(Notation notation,
+        const char *int_fmt = "{}", const char *uint_fmt = "{}",
+        const char *float_fmt = "{}",
+        const char *string_fmt = "\"{}\"", const char *ptr_fmt = "{}",
+        const char *aggregate_begin = "(", const char *aggregate_end = ")",
+        const char *aggregate_sep = ",", const char *array_begin = "{",
+        const char *array_end = "}", const char *array_sep = ",",
         int flags = escape_strings);
 };
 
 
 
 /// Return a string containing the data values formatted according
-/// to the type and the optional formatting control arguments.
+/// to the type and the optional formatting control arguments. Will be
+/// deprecated someday as printf formatting falls out of favor.
 OIIO_API std::string
 tostring(TypeDesc type, const void* data, const tostring_formatting& fmt = {});
 
@@ -496,7 +564,7 @@ tostring(TypeDesc type, const void* data, const tostring_formatting& fmt = {});
 ///   `char*`): it will always succeed, producing a string akin to calling
 ///   `tostring()`.
 /// * If dsttype is int32 or uint32: other integer types will do their best
-///   (caveat emptor if you mix signed/unsiged). Also a source string will
+///   (caveat emptor if you mix signed/unsigned). Also a source string will
 ///   convert to int if and only if its characters form a valid integer.
 /// * If dsttype is float: inteegers and other float types will do
 ///   their best conversion; strings will convert if and only if their
@@ -507,3 +575,32 @@ convert_type(TypeDesc srctype, const void* src,
 
 
 OIIO_NAMESPACE_END
+
+
+
+// Supply a fmtlib compatible custom formatter for TypeDesc.
+FMT_BEGIN_NAMESPACE
+template <>
+struct formatter<OIIO::TypeDesc> {
+    // Parses format specification
+    // C++14: constexpr auto parse(format_parse_context& ctx) {
+    auto parse(format_parse_context &ctx) -> decltype(ctx.begin()) // c++11
+    {
+        // Get the presentation type, if any. Required to be 's'.
+        auto it = ctx.begin(), end = ctx.end();
+        if (it != end && (*it == 's')) ++it;
+        // Check if reached the end of the range:
+        if (it != end && *it != '}')
+            throw format_error("invalid format");
+        // Return an iterator past the end of the parsed range:
+        return it;
+    }
+
+    template <typename FormatContext>
+    auto format(const OIIO::TypeDesc& t, FormatContext& ctx) -> decltype(ctx.out()){
+        // C++14:   auto format(const point& p, FormatContext& ctx) {
+        // ctx.out() is an output iterator to write to.
+        return format_to(ctx.out(), "{}", t.c_str());
+    }
+};
+FMT_END_NAMESPACE
