@@ -5,10 +5,7 @@
 
 #include <iostream>
 
-#include <OpenEXR/half.h>
-
 #include "imageviewer.h"
-#include <OpenImageIO/fmath.h>
 #include <OpenImageIO/strutil.h>
 
 
