@@ -2,6 +2,7 @@
 // SPDX-License-Identifier: BSD-3-Clause
 // https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
+#include <algorithm>
 #include <cmath>
 #include <memory>
 #include <string>
@@ -149,6 +150,18 @@ ColorConfig::supportsOpenColorIO()
 
 
 
+int
+ColorConfig::OpenColorIO_version_hex()
+{
+#ifdef USE_OCIO
+    return OCIO_VERSION_HEX;
+#else
+    return 0;
+#endif
+}
+
+
+
 // Hidden implementation of ColorConfig
 class ColorConfig::Impl {
 public:
@@ -1303,8 +1316,8 @@ ColorConfig::createFileTransform(ustring name, bool inverse) const
         OCIO::FileTransformRcPtr transform = OCIO::FileTransform::Create();
         transform->setSrc(name.c_str());
         transform->setInterpolation(OCIO::INTERP_BEST);
-        OCIO::TransformDirection dir = inverse ? OCIO::TRANSFORM_DIR_INVERSE
-                                               : OCIO::TRANSFORM_DIR_FORWARD;
+        OCIO::TransformDirection dir    = inverse ? OCIO::TRANSFORM_DIR_INVERSE
+                                                  : OCIO::TRANSFORM_DIR_FORWARD;
         OCIO::ConstContextRcPtr context = config->getCurrentContext();
         OCIO::ConstProcessorRcPtr p;
         try {
@@ -1335,14 +1348,67 @@ ColorConfig::createMatrixTransform(const Imath::M44f& M, bool inverse) const
 
 
 string_view
-ColorConfig::parseColorSpaceFromString(string_view str) const
+ColorConfig::getColorSpaceFromFilepath(string_view str) const
 {
-#ifdef USE_OCIO
+#if defined(USE_OCIO) && (OCIO_VERSION_HEX >= MAKE_OCIO_VERSION_HEX(2, 1, 0))
     if (getImpl() && getImpl()->config_) {
-        return getImpl()->config_->parseColorSpaceFromString(str.c_str());
+        string_view r = getImpl()->config_->getColorSpaceFromFilepath(
+            str.c_str());
+        if (!getImpl()->config_->filepathOnlyMatchesDefaultRule(str.c_str()))
+            return r;
     }
 #endif
-    return "";
+    // Fall back on parseColorSpaceFromString
+    return parseColorSpaceFromString(str);
+}
+
+
+
+string_view
+ColorConfig::parseColorSpaceFromString(string_view str) const
+{
+#if defined(USE_OCIO) && (OCIO_VERSION_HEX < MAKE_OCIO_VERSION_HEX(2, 1, 0))
+    if (getImpl() && getImpl()->config_)
+        return getImpl()->config_->parseColorSpaceFromString(str.c_str());
+#endif
+
+    // Reproduce the logic in OCIO v1 parseColorSpaceFromSring
+
+    if (str.empty())
+        return "";
+
+    // Get the colorspace names, sorted shortest-to-longest
+    auto names = getColorSpaceNames();
+    std::sort(names.begin(), names.end(),
+              [](const std::string& a, const std::string& b) {
+                  return a.length() < b.length();
+              });
+
+    // See if it matches a LUT name.
+    // This is the position of the RIGHT end of the colorspace substring,
+    // not the left
+    size_t rightMostColorPos = std::string::npos;
+    std::string rightMostColorspace;
+
+    // Find the right-most occcurance within the string for each colorspace.
+    for (auto&& csname : names) {
+        // find right-most extension matched in filename
+        size_t pos = Strutil::irfind(str, csname);
+        if (pos == std::string::npos)
+            continue;
+
+        // If we have found a match, move the pointer over to the right end
+        // of the substring.  This will allow us to find the longest name
+        // that matches the rightmost colorspace
+        pos += csname.size();
+
+        if (rightMostColorPos == std::string::npos
+            || pos >= rightMostColorPos) {
+            rightMostColorPos   = pos;
+            rightMostColorspace = csname;
+        }
+    }
+    return string_view(ustring(rightMostColorspace));
 }
 
 
@@ -1517,6 +1583,15 @@ colorconvert_impl(ImageBuf& R, const ImageBuf& A,
                     for (; !r.done(); ++r, dstPtr += 4)
                         for (int c = 0; c < channelsToCopy; ++c)
                             r[c] = dstPtr[c];
+                    if (channelsToCopy < roi.chend && (&R != &A)) {
+                        // If there are "leftover" channels, just copy them
+                        // unaltered from the source.
+                        a.rerange(roi.xbegin, roi.xend, j, j + 1, k, k + 1);
+                        r.rerange(roi.xbegin, roi.xend, j, j + 1, k, k + 1);
+                        for (; !r.done(); ++r, ++a)
+                            for (int c = channelsToCopy; c < roi.chend; ++c)
+                                r[c] = 0.5 + 10 * a[c];
+                    }
                 }
             }
         });
@@ -1614,7 +1689,6 @@ ImageBufAlgo::colorconvert(ImageBuf& dst, const ImageBuf& src,
     // If the processor is a no-op (and it's not an in-place conversion),
     // use copy() to simplify the operation.
     if (processor->isNoOp()) {
-        roi.chend = std::max(roi.chbegin + 4, roi.chend);
         logtime.stop();  // transition to copy
         return ImageBufAlgo::copy(dst, src, TypeUnknown, roi, nthreads);
     }
@@ -1657,8 +1731,8 @@ ImageBufAlgo::colorconvert(const ImageBuf& src, const ColorProcessor* processor,
 
 bool
 ImageBufAlgo::ociolook(ImageBuf& dst, const ImageBuf& src, string_view looks,
-                       string_view from, string_view to, bool inverse,
-                       bool unpremult, string_view key, string_view value,
+                       string_view from, string_view to, bool unpremult,
+                       bool inverse, string_view key, string_view value,
                        ColorConfig* colorconfig, ROI roi, int nthreads)
 {
     pvt::LoggedTimer logtime("IBA::ociolook");
@@ -1701,12 +1775,12 @@ ImageBufAlgo::ociolook(ImageBuf& dst, const ImageBuf& src, string_view looks,
 
 ImageBuf
 ImageBufAlgo::ociolook(const ImageBuf& src, string_view looks, string_view from,
-                       string_view to, bool inverse, bool unpremult,
+                       string_view to, bool unpremult, bool inverse,
                        string_view key, string_view value,
                        ColorConfig* colorconfig, ROI roi, int nthreads)
 {
     ImageBuf result;
-    bool ok = ociolook(result, src, looks, from, to, inverse, unpremult, key,
+    bool ok = ociolook(result, src, looks, from, to, unpremult, inverse, key,
                        value, colorconfig, roi, nthreads);
     if (!ok && !result.has_error())
         result.errorf("ImageBufAlgo::ociolook() error");
@@ -1775,7 +1849,7 @@ ImageBufAlgo::ociodisplay(const ImageBuf& src, string_view display,
 
 bool
 ImageBufAlgo::ociofiletransform(ImageBuf& dst, const ImageBuf& src,
-                                string_view name, bool inverse, bool unpremult,
+                                string_view name, bool unpremult, bool inverse,
                                 ColorConfig* colorconfig, ROI roi, int nthreads)
 {
     pvt::LoggedTimer logtime("IBA::ociofiletransform");
@@ -1811,11 +1885,11 @@ ImageBufAlgo::ociofiletransform(ImageBuf& dst, const ImageBuf& src,
 
 ImageBuf
 ImageBufAlgo::ociofiletransform(const ImageBuf& src, string_view name,
-                                bool inverse, bool unpremult,
+                                bool unpremult, bool inverse,
                                 ColorConfig* colorconfig, ROI roi, int nthreads)
 {
     ImageBuf result;
-    bool ok = ociofiletransform(result, src, name, inverse, unpremult,
+    bool ok = ociofiletransform(result, src, name, unpremult, inverse,
                                 colorconfig, roi, nthreads);
     if (!ok && !result.has_error())
         result.errorf("ImageBufAlgo::ociofiletransform() error");
