@@ -8,7 +8,13 @@
 #include <iostream>
 #include <string>
 
+#include <OpenImageIO/platform.h>
+
 #if USE_OPENCV
+// Suppress gcc 11 / C++20 errors about opencv 4 headers
+#    if OIIO_GNUC_VERSION >= 110000 && OIIO_CPLUSPLUS_VERSION >= 20
+#        pragma GCC diagnostic ignored "-Wdeprecated-enum-enum-conversion"
+#    endif
 #    include <opencv2/opencv.hpp>
 #endif
 
