@@ -10,6 +10,7 @@
 
 #include <OpenImageIO/dassert.h>
 #include <OpenImageIO/deepdata.h>
+#include <OpenImageIO/filesystem.h>
 #include <OpenImageIO/fmath.h>
 #include <OpenImageIO/imageio.h>
 #include <OpenImageIO/parallel.h>
@@ -73,15 +74,16 @@ ImageInput::valid_file(const std::string& filename) const
 
 
 std::unique_ptr<ImageInput>
-ImageInput::open(const std::string& filename, const ImageSpec* config)
+ImageInput::open(const std::string& filename, const ImageSpec* config,
+                 Filesystem::IOProxy* ioproxy)
 {
     if (!config) {
         // Without config, this is really just a call to create-with-open.
-        return ImageInput::create(filename, true, nullptr, std::string());
+        return ImageInput::create(filename, true, nullptr, ioproxy);
     }
 
     // With config, create without open, then try to open with config.
-    auto in = ImageInput::create(filename, false, config, std::string());
+    auto in = ImageInput::create(filename, false, config, ioproxy);
     if (!in)
         return in;  // create() failed, return the empty ptr
     ImageSpec newspec;
@@ -168,8 +170,11 @@ ImageInput::read_scanline(int y, int z, TypeDesc format, void* data,
 
     // Complex case -- either changing data type or stride
     int scanline_values = m_spec.width * m_spec.nchannels;
-    unsigned char* buf  = OIIO_ALLOCA(unsigned char,
-                                     m_spec.scanline_bytes(true));
+
+    unsigned char* buf;
+    OIIO_ALLOCATE_STACK_OR_HEAP(buf, unsigned char,
+                                m_spec.scanline_bytes(true));
+
     bool ok = read_native_scanline(current_subimage(), current_miplevel(), y, z,
                                    buf);
     if (!ok)
@@ -234,9 +239,26 @@ ImageInput::read_scanlines(int subimage, int miplevel, int ybegin, int yend,
                            int z, int chbegin, int chend, TypeDesc format,
                            void* data, stride_t xstride, stride_t ystride)
 {
-    ImageSpec spec = spec_dimensions(subimage, miplevel);  // thread-safe
-    if (spec.undefined())
+    ImageSpec spec;
+    int rps = 0;
+    {
+        // We need to lock briefly to retrieve rps from the spec
+        lock_guard lock(m_mutex);
+        if (!seek_subimage(subimage, miplevel))
+            return false;
+        // Copying the dimensions of the designated subimage/miplevel to a
+        // local `spec` means that we can release the lock!  (Calls to
+        // read_native_* will internally lock again if necessary.)
+        spec.copy_dimensions(m_spec);
+        // For scanline files, we also need one piece of metadata
+        if (!spec.tile_width)
+            rps = m_spec.get_int_attribute("tiff:RowsPerStrip", 64);
+    }
+    if (spec.image_bytes() < 1) {
+        errorfmt("Invalid image size {} x {} ({} chans)", m_spec.width,
+                 m_spec.height, m_spec.nchannels);
         return false;
+    }
 
     chend                     = clamp(chend, chbegin + 1, spec.nchannels);
     int nchans                = chend - chbegin;
@@ -252,11 +274,11 @@ ImageInput::read_scanlines(int subimage, int miplevel, int ybegin, int yend,
     stride_t zstride = AutoStride;
     spec.auto_stride(xstride, ystride, zstride, format, nchans, spec.width,
                      spec.height);
-    stride_t buffer_pixel_bytes = native ? native_pixel_bytes
-                                         : format.size() * nchans;
+    stride_t buffer_pixel_bytes    = native ? native_pixel_bytes
+                                            : format.size() * nchans;
     stride_t buffer_scanline_bytes = native ? native_scanline_bytes
                                             : buffer_pixel_bytes * spec.width;
-    bool contiguous = (xstride == (stride_t)buffer_pixel_bytes
+    bool contiguous                = (xstride == (stride_t)buffer_pixel_bytes
                        && ystride == (stride_t)buffer_scanline_bytes);
 
     if (native && contiguous) {
@@ -272,8 +294,8 @@ ImageInput::read_scanlines(int subimage, int miplevel, int ybegin, int yend,
 
     // Split into reasonable chunks -- try to use around 64 MB, but
     // round up to a multiple of the TIFF rows per strip (or 64).
-    int rps   = spec.get_int_attribute("tiff:RowsPerStrip", 64);
     int chunk = std::max(1, (1 << 26) / int(spec.scanline_bytes(true)));
+    chunk     = std::max(chunk, int(oiio_read_chunk));
     chunk     = round_to_multiple(chunk, rps);
     std::unique_ptr<char[]> buf(new char[chunk * native_scanline_bytes]);
 
@@ -574,19 +596,19 @@ ImageInput::read_tiles(int subimage, int miplevel, int xbegin, int xend,
     }
 
     // No such luck.  Just punt and read tiles individually.
-    bool ok            = true;
-    stride_t pixelsize = native_data ? native_pixel_bytes
-                                     : (format.size() * nchans);
-    stride_t native_pixelsize = spec.pixel_bytes(true);
-    stride_t full_pixelsize   = native_data ? native_pixelsize
-                                          : (format.size() * spec.nchannels);
+    bool ok                        = true;
+    stride_t pixelsize             = native_data ? native_pixel_bytes
+                                                 : (format.size() * nchans);
+    stride_t native_pixelsize      = spec.pixel_bytes(true);
+    stride_t full_pixelsize        = native_data ? native_pixelsize
+                                                 : (format.size() * spec.nchannels);
     stride_t full_tilewidthbytes   = full_pixelsize * spec.tile_width;
     stride_t full_tilewhbytes      = full_tilewidthbytes * spec.tile_height;
     stride_t full_tilebytes        = full_tilewhbytes * spec.tile_depth;
     stride_t full_native_tilebytes = spec.tile_bytes(true);
     size_t prefix_bytes = native_data ? spec.pixel_bytes(0, chbegin, true)
                                       : format.size() * chbegin;
-    bool allchans = (chbegin == 0 && chend == spec.nchannels);
+    bool allchans       = (chbegin == 0 && chend == spec.nchannels);
     std::vector<char> buf;
     for (int z = zbegin; z < zend; z += std::max(1, spec.tile_depth)) {
         int zd      = std::min(zend - z, spec.tile_depth);
@@ -679,8 +701,8 @@ ImageInput::read_tiles(int subimage, int miplevel, int xbegin, int xend,
 
 
 bool
-ImageInput::read_native_tile(int subimage, int miplevel, int x, int y, int z,
-                             void* data)
+ImageInput::read_native_tile(int /*subimage*/, int /*miplevel*/, int /*x*/,
+                             int /*y*/, int /*z*/, void* /*data*/)
 {
     // The base class read_native_tile fails. A format reader that supports
     // tiles MUST overload this virtual method that reads a single tile
@@ -844,6 +866,7 @@ ImageInput::read_image(int subimage, int miplevel, int chbegin, int chend,
     ImageSpec spec;
     int rps = 0;
     {
+        // We need to lock briefly to retrieve rps from the spec
         lock_guard lock(m_mutex);
         if (!seek_subimage(subimage, miplevel))
             return false;
@@ -855,6 +878,11 @@ ImageInput::read_image(int subimage, int miplevel, int chbegin, int chend,
         if (!spec.tile_width)
             rps = m_spec.get_int_attribute("tiff:RowsPerStrip", 64);
     }
+    if (spec.image_bytes() < 1) {
+        errorfmt("Invalid image size {} x {} ({} chans)", m_spec.width,
+                 m_spec.height, m_spec.nchannels);
+        return false;
+    }
 
     if (chend < 0)
         chend = spec.nchannels;
@@ -924,9 +952,10 @@ ImageInput::read_image(int subimage, int miplevel, int chbegin, int chend,
 
 
 bool
-ImageInput::read_native_deep_scanlines(int subimage, int miplevel, int ybegin,
-                                       int yend, int z, int chbegin, int chend,
-                                       DeepData& deepdata)
+ImageInput::read_native_deep_scanlines(int /*subimage*/, int /*miplevel*/,
+                                       int /*ybegin*/, int /*yend*/, int /*z*/,
+                                       int /*chbegin*/, int /*chend*/,
+                                       DeepData& /*deepdata*/)
 {
     return false;  // default: doesn't support deep images
 }
@@ -934,10 +963,11 @@ ImageInput::read_native_deep_scanlines(int subimage, int miplevel, int ybegin,
 
 
 bool
-ImageInput::read_native_deep_tiles(int subimage, int miplevel, int xbegin,
-                                   int xend, int ybegin, int yend, int zbegin,
-                                   int zend, int chbegin, int chend,
-                                   DeepData& deepdata)
+ImageInput::read_native_deep_tiles(int /*subimage*/, int /*miplevel*/,
+                                   int /*xbegin*/, int /*xend*/, int /*ybegin*/,
+                                   int /*yend*/, int /*zbegin*/, int /*zend*/,
+                                   int /*chbegin*/, int /*chend*/,
+                                   DeepData& /*deepdata*/)
 {
     return false;  // default: doesn't support deep images
 }
@@ -978,7 +1008,7 @@ ImageInput::read_native_deep_image(int subimage, int miplevel,
 
 
 int
-ImageInput::send_to_input(const char* format, ...)
+ImageInput::send_to_input(const char* /*format*/, ...)
 {
     // FIXME -- I can't remember how this is supposed to work
     return 0;
@@ -987,7 +1017,7 @@ ImageInput::send_to_input(const char* format, ...)
 
 
 int
-ImageInput::send_to_client(const char* format, ...)
+ImageInput::send_to_client(const char* /*format*/, ...)
 {
     // FIXME -- I can't remember how this is supposed to work
     return 0;
