@@ -279,13 +279,10 @@ ImageOutput::to_native_rectangle(int xbegin, int xend, int ybegin, int yend,
     // native_pixel_bytes is the size of a pixel in the FILE, including
     // the per-channel format, if specified when the file was opened.
     stride_t native_pixel_bytes = (stride_t)m_spec.pixel_bytes(true);
-    // perchanfile is true if the file has different per-channel formats
+    // perchanfile is true if the spec has different per-channel data types
+    // and the file format supports that feature.
     bool perchanfile = m_spec.channelformats.size()
                        && supports("channelformats");
-    // It's an error to pass per-channel data formats to a writer that
-    // doesn't support it.
-    if (m_spec.channelformats.size() && !perchanfile)
-        return NULL;
     // native_data is true if the user is passing data in the native format
     bool native_data           = (format == TypeDesc::UNKNOWN
                         || (format == m_spec.format && !perchanfile));
@@ -368,7 +365,7 @@ ImageOutput::to_native_rectangle(int xbegin, int xend, int ybegin, int yend,
     imagesize_t contiguoussize = contiguous
                                      ? 0
                                      : rectangle_values * input_pixel_bytes;
-    contiguoussize = (contiguoussize + 3)
+    contiguoussize             = (contiguoussize + 3)
                      & (~3);  // Round up to 4-byte boundary
     OIIO_DASSERT((contiguoussize & 3) == 0);
     imagesize_t floatsize = rectangle_values * sizeof(float);
