@@ -606,6 +606,13 @@ write_mipmap(ImageBufAlgo::MakeTextureMode mode, std::shared_ptr<ImageBuf>& img,
     ImageSpec outspec      = outspec_template;
     outspec.set_format(outputdatatype);
 
+    // Going from float to half is prone to generating Inf values if we had
+    // any floats that were out of the range that half can represent. Nobody
+    // wants Inf in textures; better to clamp.
+    bool clamp_half = (outspec.format == TypeHalf
+                       && (img->spec().format == TypeFloat
+                           || img->spec().format == TypeHalf));
+
     if (mipmap && !out->supports("multiimage") && !out->supports("mipmap")) {
         outstream << "maketx ERROR: \"" << outputfilename
                   << "\" format does not support multires images\n";
@@ -673,6 +680,11 @@ write_mipmap(ImageBufAlgo::MakeTextureMode mode, std::shared_ptr<ImageBuf>& img,
         outstream << "  Top level is " << formatres(outspec) << std::endl;
     }
 
+    if (clamp_half) {
+        std::shared_ptr<ImageBuf> tmp(new ImageBuf);
+        ImageBufAlgo::clamp(*tmp, *img, -HALF_MAX, HALF_MAX, true);
+        std::swap(tmp, img);
+    }
     if (!img->write(out)) {
         // ImageBuf::write transfers any errors from the ImageOutput to
         // the ImageBuf.
@@ -810,6 +822,8 @@ write_mipmap(ImageBufAlgo::MakeTextureMode mode, std::shared_ptr<ImageBuf>& img,
                     Filter2D::destroy(filter);
                 }
             }
+            if (clamp_half)
+                ImageBufAlgo::clamp(*small, *small, -HALF_MAX, HALF_MAX, true);
 
             stat_miptime += miptimer();
             outspec = smallspec;
@@ -995,6 +1009,11 @@ make_texture_impl(ImageBufAlgo::MakeTextureMode mode, const ImageBuf* input,
     }
     OIIO_DASSERT(src.get());
 
+    if (src->deep()) {
+        outstream << "Deep images cannot be turned into textures.\n";
+        return false;
+    }
+
     if (!outputfilename.length()) {
         std::string fn = src->name();
         if (fn.length()) {
