@@ -19,6 +19,7 @@
 #endif
 
 #include <boost/algorithm/string.hpp>
+#include <boost/algorithm/string/find.hpp>
 
 #include <OpenImageIO/dassert.h>
 #include <OpenImageIO/platform.h>
@@ -191,7 +192,7 @@ Strutil::vsprintf(const char* fmt, va_list ap)
 #ifdef va_copy
         va_copy(ap, apsave);
 #else
-        ap     = apsave;
+        ap = apsave;
 #endif
     }
 }
@@ -465,6 +466,44 @@ Strutil::icontains(string_view a, string_view b)
 }
 
 
+size_t
+Strutil::find(string_view a, string_view b)
+{
+    return a.find(b);
+}
+
+
+size_t
+Strutil::rfind(string_view a, string_view b)
+{
+    return a.rfind(b);
+}
+
+
+size_t
+Strutil::ifind(string_view a, string_view b)
+{
+    if (a.empty())
+        return string_view::npos;
+    if (b.empty())
+        return 0;
+    auto f = boost::algorithm::ifind_first(a, b, std::locale::classic());
+    return f.empty() ? string_view::npos : f.begin() - a.data();
+}
+
+
+size_t
+Strutil::irfind(string_view a, string_view b)
+{
+    if (a.empty())
+        return string_view::npos;
+    if (b.empty())
+        return a.size();
+    auto f = boost::algorithm::ifind_last(a, b, std::locale::classic());
+    return f.empty() ? string_view::npos : f.begin() - a.data();
+}
+
+
 void
 Strutil::to_lower(std::string& a)
 {
@@ -1114,8 +1153,8 @@ decode(uint32_t* state, uint32_t* codep, uint32_t byte)
 {
     uint32_t type = utf8d[byte];
     *codep        = (*state != UTF8_ACCEPT) ? (byte & 0x3fu) | (*codep << 6)
-                                     : (0xff >> type) & (byte);
-    *state = utf8d[256 + *state + type];
+                                            : (0xff >> type) & (byte);
+    *state        = utf8d[256 + *state + type];
     return *state;
 }
 
