@@ -61,6 +61,7 @@
 #include <OpenImageIO/dassert.h>
 #include <OpenImageIO/strutil.h>
 #include <OpenImageIO/sysutil.h>
+#include <OpenImageIO/ustring.h>
 
 #include <boost/version.hpp>
 #if BOOST_VERSION >= 106500
@@ -270,10 +271,22 @@ Sysutil::this_program_path()
 
 
 
+string_view
+Sysutil::getenv(string_view name, string_view defaultval)
+{
+    const char* env = ::getenv(std::string(name).c_str());
+    if (!env && defaultval.size())
+        env = ustring(defaultval).c_str();
+    return string_view(env ? env : "");
+}
+
+
+
 string_view
 Sysutil::getenv(string_view name)
 {
-    return string_view(::getenv(name.c_str()));
+    const char* env = ::getenv(std::string(name).c_str());
+    return string_view(env ? env : "");
 }
 
 
