@@ -59,6 +59,7 @@ OIIO_GCC_PRAGMA(GCC diagnostic ignored "-Wunused-parameter")
 #include <OpenEXR/ImfDoubleAttribute.h>
 #include <OpenEXR/ImfEnvmapAttribute.h>
 #include <OpenEXR/ImfFloatAttribute.h>
+#include <OpenEXR/ImfHeader.h>
 #if OPENEXR_HAS_FLOATVECTOR
 #    include <OpenEXR/ImfFloatVectorAttribute.h>
 #endif
@@ -112,12 +113,21 @@ public:
             throw Iex::IoExc("Unexpected end of file.");
         return n;
     }
+#if OIIO_USING_IMATH >= 3
+    virtual uint64_t tellg() { return m_io->tell(); }
+    virtual void seekg(uint64_t pos)
+    {
+        if (!m_io->seek(pos))
+            throw Iex::IoExc("File input failed.");
+    }
+#else
     virtual Imath::Int64 tellg() { return m_io->tell(); }
     virtual void seekg(Imath::Int64 pos)
     {
         if (!m_io->seek(pos))
             throw Iex::IoExc("File input failed.");
     }
+#endif
     virtual void clear() {}
 
 private:
