@@ -48,6 +48,7 @@ OIIO_GCC_PRAGMA(GCC diagnostic ignored "-Wunused-parameter")
 #if OPENEXR_HAS_FLOATVECTOR
 #    include <OpenEXR/ImfFloatVectorAttribute.h>
 #endif
+#include <OpenEXR/ImfHeader.h>
 #include <OpenEXR/ImfIntAttribute.h>
 #include <OpenEXR/ImfKeyCodeAttribute.h>
 #include <OpenEXR/ImfMatrixAttribute.h>
@@ -56,6 +57,7 @@ OIIO_GCC_PRAGMA(GCC diagnostic ignored "-Wunused-parameter")
 #include <OpenEXR/ImfTimeCodeAttribute.h>
 #include <OpenEXR/ImfVecAttribute.h>
 
+#include <OpenEXR/ImfDeepFrameBuffer.h>
 #include <OpenEXR/ImfDeepScanLineOutputPart.h>
 #include <OpenEXR/ImfDeepTiledOutputPart.h>
 #include <OpenEXR/ImfDoubleAttribute.h>
@@ -94,12 +96,21 @@ public:
         if (m_io->write(c, n) != size_t(n))
             throw Iex::IoExc("File output failed.");
     }
+#if OIIO_USING_IMATH >= 3
+    virtual uint64_t tellp() { return m_io->tell(); }
+    virtual void seekp(uint64_t pos)
+    {
+        if (!m_io->seek(pos))
+            throw Iex::IoExc("File output failed.");
+    }
+#else
     virtual Imath::Int64 tellp() { return m_io->tell(); }
     virtual void seekp(Imath::Int64 pos)
     {
         if (!m_io->seek(pos))
             throw Iex::IoExc("File output failed.");
     }
+#endif
 
 private:
     Filesystem::IOProxy* m_io = nullptr;
