@@ -477,7 +477,7 @@ const PSDInput::ResourceLoader PSDInput::resource_loaders[]
 
 const char* PSDInput::additional_info_psb[]
     = { "LMsk", "Lr16", "Lr32", "Layr", "Mt16", "Mt32", "Mtrn",
-        "Alph", "FMsk", "Ink2", "FEid", "FXid", "PxSD" };
+        "Alph", "FMsk", "Ink2", "FEid", "FXid", "PxSD", "cinf" };
 
 const unsigned int PSDInput::additional_info_psb_count
     = sizeof(additional_info_psb) / sizeof(additional_info_psb[0]);
@@ -1787,8 +1787,8 @@ PSDInput::setup()
     for (Layer& layer : m_layers) {
         spec_channel_count = m_WantRaw ? mode_channel_count[m_header.color_mode]
                                        : 3;
-        raw_channel_count = mode_channel_count[m_header.color_mode];
-        bool transparency = (bool)layer.channel_id_map.count(
+        raw_channel_count  = mode_channel_count[m_header.color_mode];
+        bool transparency  = (bool)layer.channel_id_map.count(
             ChannelID_Transparency);
         if (transparency) {
             spec_channel_count++;
