@@ -7,7 +7,7 @@
 
 // Python.h uses the 'register' keyword, don't warn about it being
 // deprecated in C++17.
-#if (__cplusplus >= 201703L && defined(__GNUC__))
+#if (__cplusplus >= 201703L && defined(__GNUC__)) || defined(__clang__)
 #    pragma GCC diagnostic ignored "-Wregister"
 #endif
 
@@ -36,6 +36,10 @@
 #include <OpenImageIO/span.h>
 #include <OpenImageIO/typedesc.h>
 
+#if PY_MAJOR_VERSION < 3
+OIIO_CLANG_PRAGMA(GCC diagnostic ignored "-Wunused-value")
+#endif
+
 #include <pybind11/numpy.h>
 #include <pybind11/operators.h>
 #include <pybind11/pybind11.h>
@@ -593,9 +597,7 @@ case TypeDesc::TYPE:                                                       \
         ParamValue_convert_dispatch(INT);
         // ParamValue_convert_dispatch(ULONGLONG);
         // ParamValue_convert_dispatch(LONGLONG);
-#ifdef _HALF_H_
         ParamValue_convert_dispatch(HALF);
-#endif
         ParamValue_convert_dispatch(FLOAT);
         ParamValue_convert_dispatch(DOUBLE);
     case TypeDesc::STRING:
