@@ -3,6 +3,7 @@
 // https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 #include "sgi_pvt.h"
 #include <OpenImageIO/dassert.h>
+#include <OpenImageIO/fmath.h>
 
 OIIO_PLUGIN_NAMESPACE_BEGIN
 
@@ -114,7 +115,7 @@ SgiInput::open(const std::string& name, ImageSpec& spec)
 
 
 bool
-SgiInput::read_native_scanline(int subimage, int miplevel, int y, int z,
+SgiInput::read_native_scanline(int subimage, int miplevel, int y, int /*z*/,
                                void* data)
 {
     lock_guard lock(m_mutex);
