@@ -151,8 +151,8 @@ SoftimageInput::open(const std::string& name, ImageSpec& spec)
 
 
 bool
-SoftimageInput::read_native_scanline(int subimage, int miplevel, int y, int z,
-                                     void* data)
+SoftimageInput::read_native_scanline(int subimage, int miplevel, int y,
+                                     int /*z*/, void* data)
 {
     lock_guard lock(m_mutex);
     if (!seek_subimage(subimage, miplevel))
