@@ -634,7 +634,7 @@ TIFFOutput::open(const std::string& name, const ImageSpec& userspec,
     if (icc_profile_parameter != NULL) {
         unsigned char* icc_profile
             = (unsigned char*)icc_profile_parameter->data();
-        uint32 length = icc_profile_parameter->type().size();
+        uint32_t length = icc_profile_parameter->type().size();
         if (icc_profile && length)
             TIFFSetField(m_tif, TIFFTAG_ICCPROFILE, length, icc_profile);
     }
@@ -906,7 +906,11 @@ TIFFOutput::write_exif_data()
     }
 
     // Now write the directory of Exif data
-    uint64 dir_offset = 0;
+#    ifndef TIFF_GCC_DEPRECATED
+    uint64 dir_offset = 0;  // old type
+#    else
+    uint64_t dir_offset = 0;
+#    endif
     if (!TIFFWriteCustomDirectory(m_tif, &dir_offset)) {
         errorf("failed TIFFWriteCustomDirectory() of the Exif data");
         return false;
@@ -962,6 +966,7 @@ rgb_to_cmyk(int n, const T* rgb, size_t rgb_stride, T* cmyk, size_t cmyk_stride)
         float one_minus_K     = std::max(R, std::max(G, B));
         float one_minus_K_inv = (one_minus_K <= 1e-6) ? 0.0f
                                                       : 1.0f / one_minus_K;
+
         float C = (one_minus_K - R) * one_minus_K_inv;
         float M = (one_minus_K - G) * one_minus_K_inv;
         float Y = (one_minus_K - B) * one_minus_K_inv;
@@ -1168,12 +1173,13 @@ TIFFOutput::write_scanlines(int ybegin, int yend, int z, TypeDesc format,
     for (size_t stripidx = 0; y + m_rowsperstrip <= yend;
          y += m_rowsperstrip, ++stripidx) {
         char* cbuf = compressed_scratch.get() + stripidx * cbound;
+        auto out   = this;
         tasks.push(pool->push([=, &ok](int /*id*/) {
             memcpy((void*)data, origdata, strip_bytes);
-            this->compress_one_strip((void*)data, strip_bytes, cbuf, cbound,
-                                     this->m_spec.nchannels, this->m_spec.width,
-                                     m_rowsperstrip, compressed_len + stripidx,
-                                     &ok);
+            out->compress_one_strip((void*)data, strip_bytes, cbuf, cbound,
+                                    out->m_spec.nchannels, out->m_spec.width,
+                                    out->m_rowsperstrip,
+                                    compressed_len + stripidx, &ok);
         }));
         data     = (char*)data + strip_bytes;
         origdata = (char*)origdata + strip_bytes;
