@@ -1,5 +1,10 @@
 #!/usr/bin/env python
 
+from __future__ import absolute_import
+import os
+
+oiiotoolsrcdir = os.path.join(OIIO_TESTSUITE_ROOT, "oiiotool", "src")
+
 # Make test pattern with increasing intensity left to right, decreasing
 # alpha going down. Carefully done so that the first pixel is 0.0, last
 # pixel is 1.0 (correcting for the half pixel offset).
@@ -8,9 +13,11 @@ command += oiiotool ("-pattern fill:topleft=0,0,0,1:topright=1,1,1,1:bottomleft=
 
 
 # test --colormap
-command += oiiotool ("--autocc ../oiiotool/src/tahoe-tiny.tif --colormap inferno "
+command += oiiotool ("--autocc " + oiiotoolsrcdir+"/tahoe-tiny.tif" +
+                     " --colormap inferno "
             + "-d uint8 -o colormap-inferno.tif")
-command += oiiotool ("--autocc ../oiiotool/src/tahoe-tiny.tif --colormap .25,.25,.25,0,.5,0,1,0,0 "
+command += oiiotool ("--autocc " + oiiotoolsrcdir+"/tahoe-tiny.tif" +
+                     " --colormap .25,.25,.25,0,.5,0,1,0,0 "
             + "-d uint8 -o colormap-custom.tif")
 
 # test unpremult/premult
@@ -24,16 +31,16 @@ command += oiiotool ("unpremult.exr --premult -o premult.exr")
 command += oiiotool ("--no-autopremult src/rgba.tga --ch R,G,B -o rgbfromtga.png")
 
 # test --contrast
-command += oiiotool ("--autocc ../oiiotool/src/tahoe-tiny.tif "
-                     + "-contrast:black=0.1:white=0.75 -d uint8 -o contrast-stretch.tif")
-command += oiiotool ("--autocc ../oiiotool/src/tahoe-tiny.tif "
-                     + "-contrast:min=0.1:max=0.75 -d uint8 -o contrast-shrink.tif")
-command += oiiotool ("--autocc ../oiiotool/src/tahoe-tiny.tif "
-                     + "-contrast:black=1:white=0 -d uint8 -o contrast-inverse.tif")
-command += oiiotool ("--autocc ../oiiotool/src/tahoe-tiny.tif "
-                     + "-contrast:black=1,1,.25:white=1,1,0.25 -d uint8 -o contrast-threshold.tif")
-command += oiiotool ("--autocc ../oiiotool/src/tahoe-tiny.tif "
-                     + "-contrast:scontrast=5 -d uint8 -o contrast-sigmoid5.tif")
+command += oiiotool ("--autocc " + oiiotoolsrcdir+"/tahoe-tiny.tif" +
+                     " -contrast:black=0.1:white=0.75 -d uint8 -o contrast-stretch.tif")
+command += oiiotool ("--autocc " + oiiotoolsrcdir+"/tahoe-tiny.tif" +
+                     " -contrast:min=0.1:max=0.75 -d uint8 -o contrast-shrink.tif")
+command += oiiotool ("--autocc " + oiiotoolsrcdir+"/tahoe-tiny.tif" +
+                     " -contrast:black=1:white=0 -d uint8 -o contrast-inverse.tif")
+command += oiiotool ("--autocc " + oiiotoolsrcdir+"/tahoe-tiny.tif" +
+                     " -contrast:black=1,1,.25:white=1,1,0.25 -d uint8 -o contrast-threshold.tif")
+command += oiiotool ("--autocc " + oiiotoolsrcdir+"/tahoe-tiny.tif" +
+                     " -contrast:scontrast=5 -d uint8 -o contrast-sigmoid5.tif")
 
 
 
@@ -50,7 +57,8 @@ command += oiiotool ("greyalpha_linear.tif --colorconvert:unpremult=1 linear sRG
 command += oiiotool ("greyalpha_linear.tif --colorconvert:unpremult=1 linear Cineon -o greyalpha_Cineon_un.tif")
 
 # test color convert by matrix
-command += oiiotool ("--autocc ../oiiotool/src/tahoe-tiny.tif "
+command += oiiotool ("--autocc " + oiiotoolsrcdir+"/tahoe-tiny.tif"+
+                     " "
                      + "--ccmatrix 0.805,0.506,-0.311,0,-0.311,0.805,0.506,0,0.506,-0.311,0.805,0,0,0,0,1 "
                      + "-d uint8 -o tahoe-ccmatrix.tif")
 
