@@ -9,20 +9,20 @@
 # Some tests to verify that we are transferring data formats properly.
 #
 command += oiiotool ("-pattern checker 128x128 3 -d uint8 -tile 16 16 -o uint8.tif " +
-                     "-echo '\nexplicit -d uint save result: ' -metamatch \"width|tile\" -i:info=2 uint8.tif")
+                     "-echo \"explicit -d uint save result: \" -metamatch \"width|tile\" -i:info=2 uint8.tif -echo \"\"")
 # Un-modified copy should preserve data type and tiling
 command += oiiotool ("uint8.tif -o tmp.tif " +
-                     "-echo '\nunmodified copy result: ' -metamatch \"width|tile\" -i:info=2 tmp.tif")
+                     "-echo \"unmodified copy result: \" -metamatch \"width|tile\" -i:info=2 tmp.tif -echo \"\"")
 # Copy with explicit data request should change data type
 command += oiiotool ("uint8.tif -d uint16 -o copy_uint16.tif " +
-                     "-echo '\ncopy with explicit -d uint16 result: ' -metamatch \"width|tile\" -i:info=2 copy_uint16.tif")
+                     "-echo \"copy with explicit -d uint16 result: \" -metamatch \"width|tile\" -i:info=2 copy_uint16.tif -echo \"\"")
 # Subimage concatenation should preserve data type
 command += oiiotool ("uint8.tif copy_uint16.tif -siappend -o tmp.tif " +
-                     "-echo '\nsiappend result: ' -metamatch \"width|tile\" -i:info=2 tmp.tif")
+                     "-echo \"siappend result: \" -metamatch \"width|tile\" -i:info=2 tmp.tif -echo \"\"")
 # Combining two images preserves the format of the first read input, if
 # there are not any other hints:
 command += oiiotool ("-pattern checker 128x128 3 uint8.tif -add -o tmp.tif " +
-                     "-echo '\ncombining images result: ' -metamatch \"width|tile\" -i:info=2 tmp.tif")
+                     "-echo \"combining images result: \" -metamatch \"width|tile\" -i:info=2 tmp.tif -echo \"\"")
 
 # test --crop
 command += oiiotool (OIIO_TESTSUITE_IMAGEDIR + "/grid.tif --crop 100x400+50+200 -o crop.tif")
